// Blinkende Status-LED mit 0-10 Lichtimpulsen alle 10 Sekunden
//   verwendet zwei PWM-'slices', einen für 5Hz-Blinken/Impulsen, zweiter als Input/Output, dass nur x Signale kommen und dann 10 Sekunden Pause
//   Die LED hängt dann zwischen A-Output des ersten PWM-slice und dem A-Output des zweiten PWM-slice
//
// Dieses Beispiel verwendet die Pico-Arduino Boarderweiterung von Earle Philhower (Version 2.6.3)
// nach Anregung des Buches "RPi Pico Schaltungen und Projekte" von Burkhard Kainka, Seite 5
// Einbindung über die Boardverwalter-URL https://github.com/earlephilhower/arduino-pico/releases/download/global/package_rp2040_index.json
// unter "Datei | Voreinstellungen" in der Arduino-IDE
//
// Im Buch auf Seite 18 (PWM-Steuerung), Seite 56-75 stehen Infos zu PWM, insbesondere ab Seite 74 zu PWM als Input für Frequenzteiler 
// Die Zuordung der GPIO zu PWM 0-7 steht auf Seite 13 von https://datasheets.raspberrypi.com/rp2040/rp2040-datasheet.pdf
//
// Die Library pwm ist beschrieben in:
//      /home/username/.arduino15/packages/rp2040/hardware/rp2040/2.6.3/pico-sdk/src/rp2_common/hardware_pwm/include/hardware/pwm.h
// Hinweise für PWM-Nutzung mit dem PiPico gibt es auch unter
//      https://www.i-programmer.info/programming/hardware/14849-the-pico-in-c-basic-pwm.html?start=1.
//   und mit mehr Details im Buch
//      Programming The Raspberry Pi Pico/W In C, Second Edition  von Harry Fairhead
//
//  Timing-Übersicht:
//  Blink-Kanal (0):   X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_X_
//  Burst-Kanal (1):   XXXXXXXXXX____________________________________________________________________
//  LED leuchtet       X_X_X_X_X_____________________________________________________________________
// 
//  So sind die Parameter zugeordnet:
//  slice_blink_wrap : ==
//  slice_blink_level: =
//  slice_burst_wrap : ==============================================================================
//  slice_burst_level: ==========
//
//  So ist die logische Reihenfolge (nur die Verbindung GPIO-0 zu GPIO-3 und der Anschluss an Diode sind in Hardware):
//
//            Clock               -> divider -> slice_blink-Counter(wrap) -> slice_blink-A-Output(level) -> GPIO-0 -- LED-Anode
//      ______________________________________________________________________________________________________|
//     |
//  GPIO-3 -> slice_burst-B-Input -> divider -> slice_burst-Counter(wrap) -> slice_burst-B-Output(level) -> GPIO-2 -- LED-Kathode 
//
//  PWM ist eigentlich für hohe Frequenzen gemacht. Wenn man die maximalen Werte für die Teiler ausnutzt, können 3,7Hz als kleinste Frequenz erreicht werden
//    bei 3,7Hz kann man die Impulse gerade noch so mit dem Auge "zählen", bei 5Hz wird es schon eher "gefühlt"
//  Es gibt noch die Möglichkeit, die Systemfrequenz zu reduzieren. Wenn man von den regulären 125MHZ auf 48MHz wechselt, ist die Blink-Frequenz 1,4Hz


#ifndef PWM_STATUS
#define PWM_STATUS
#include <Arduino.h> 
#include "hardware/pwm.h"

class PWM_Status {
  private:
    uint                  _LED_Anode;                                    // Slice_Blink, Output A
    uint                  _LED_Kathode;                                  // Slice_Burst, Output A
    uint                  _slice_burst_Input;                            // Slice_Burst, Input  B, Muss physisch mit LED_Anode verbunden werden
    uint                  _slice_blink;                                                     
    uint                  _channel_blink;    
    uint                  _slice_burst;                                                
    uint                  _channel_burst;   
    static const uint8_t  _slice_blink_divider = 255;                    // Höchster möglicher Wert wegen uint_8,  reduziert 125MHz auf 490kHz (-> 3,7Hz) oder 48MHz auf 188kHz  (->1,4Hz)
    static const uint16_t _slice_blink_wrap    = 65534;                  // Höchster möglicher Wert wegen uint_16, reduziert            490kHz auf 7,5Hz  oder           188kHz auf 2,9Hz
                                                                         //  Durch Phase-Correct-Mode werden es dann bei     125MHz     ->         3,7Hz oder bei  48MHz            1,4Hz  
    static const uint16_t _slice_blink_level   = 13000;                  // Setzt den Duty-Cycle ("Tastgrad"), hier 2/10    
    static const uint8_t  _slice_burst_divider = 1;                      // Hier kein Divider
                                                                         // "static" führt dazu, dass die "const"-Variable eine "Klassenkonstante" wird. 
                                                                         //     Das benötigt etwas weniger Programmspeicher, wenn mehrere Objekte definiert werden.
                                                                         //     Folgt Tipp2 aus dem Buch Effektiv C++ programmieren, 3. Auflage, von Scott Meyers
    uint16_t              _slice_burst_wrap;                             // Gibt die Anzahl der Taktzyklen an, bis der Burst Modus wieder beginnt
    uint16_t              _slice_burst_level;                            // Anzahl blinken
/*
    PWM_Status(const PWM_Status&);                                       // Indem der Kopierkonstruktruktor privat deklariert wird, erstellt der Compiler keine public-Funktion
                                                                         //    damit wird verhindert dass z.B. folgendes programmiert werden kann  PWM_Status TestDopplung(StatusLED);
    PWM_Status& operator=(const PWM_Status&);                            // Indem der Kopierzuweisungsoperator privat deklariert wird, erstellt der Compiler keine public-Funktion
                                                                         //    damit wird verhindert dass z.B. folgendes programmiert werden kann  PWM_Status TestDopplung = StatusLED;
                                                                         // Folgt Tipp 6 aus dem Buch Effektiv C++ programmieren, 3. Auflage, von Scott Meyers
*/
  public:
    PWM_Status(const uint LED_Anode, const uint LED_Kathode, const uint slice_burst_Input, 
                 const uint16_t Wiederholung, uint16_t Level );
    void init();
    void start();
    void Status(uint16_t NewLevel);
};
#endif